@@ -16,8 +16,6 @@ import Text.Blaze.Html.Renderer.Text (renderHtml)
 import qualified Data.ByteString.Lazy as B
 import qualified Data.ByteString.Char8 as BS
 import System.FilePath ((</>))
-import Prelude ()
-import Prelude.Compat
 
 main :: IO ()
 main = scotty 3000 $ do
